/**
 * jQuery acms-latlngmapper plugin 1.0 (2011-06-28)
 *
 * Copyright (c) 2011 appleple.com
 *
 * @example
 *
 *  ■ JS
 *    a-blog cmsのindex.jsを利用しない場合は、別途Google Loaderを読み込みます
 *
 *      <script type="text/javascript">
 *      google.load('maps', '3', {
 *       'callback'        : function()
 *          {
 *            $('#map').acmsLatlngMapper({
 *                  mark_pool : '#js_mapItems li',
 *                  mark_reset: '#reset'
 *              });
 *          },
 *          other_params : 'sensor=false'
 *      });
 *      </script>
 *
 *  ■ HTML
 *    緯度・経度・画像パスの指定はカスタムフィールドなどを利用します
 *
 *      <ul id="js_mapItems" style="display:none;">
 *          <!-- BEGIN_MODULE Entry_Summary --><!-- BEGIN entry:loop -->
 *          <li data-lat="{緯度}" data-lng="{経度}" data-icon="{マーカーアイコン画像のパス}">
 *              <div>
 *                  <a href="{url}">{title}</a>
 *              </div>
 *          </li><!-- END entry:loop -->
 *          <!-- END_MODULE Entry_Summary -->
 *      </ul>
 *      <p><a id="reset" href="#">地図をはじめの状態にもどす</a></p>
 */
(function($) {
$.fn.acmsLatlngMapper = function(options) {
    var defaults = {
            option_map: {
                scrollwheel : false
            },
            option_info  : {},
            option_marker: {},
            mark_pool : '',
            mark_reset: '',
            data_lat  : 'data-lat',
            data_lng  : 'data-lng',
            data_mark : 'data-mark',
            data_icon : 'data-icon',
            data_content: 'data-content',
            markerClickInfo : true,
            markerClickZoom : true,
            itemClickInfo   : true,
            itemClickZoom   : true,
            zoomToLevel : 13
        },
        settings = $.extend({}, defaults, options);

    return this.each(function() {
        var mapper = new LatlngMapper();
        mapper.init(this, settings);
        this.LatlngMapperInitialized = true;
    });
};

var LatlngMapper = function() {
    return this;
};

LatlngMapper.prototype.init = function(elm, settings) {

    if ( !!elm.LatlngMapperInitialized ) { return true; }

    var $map, $items, self,
        GMap, GInfoWin,
        minLat, minLng, maxLat, maxLng,
        dataLat, dataLng, dataMark, dataIcon, dataContent;

    // initialize
    $items  = $(settings.mark_pool);
    $map    = $(elm);
    self    = this;
    self.markers = {};

    GMap    = this.GMap     = new google.maps.Map($map.get(0), $.extend({
        'zoom'       : 10,
        'mapTypeId'  : google.maps.MapTypeId.ROADMAP
    }, settings.option_map));

    GInfoWin= this.GInfoWin = new google.maps.InfoWindow(settings.option_info);

    // center
    minLat = minLng = maxLat = maxLng = null;

    // data-*
    dataLat  = settings.data_lat;
    dataLng  = settings.data_lng;
    dataMark = settings.data_mark;
    dataIcon = settings.data_icon;
    dataContent = settings.data_content;

    // markers
    $items.each(function(i) {
        var $i  = $(this),
            lat = $i.attr(dataLat),
            lng = $i.attr(dataLng),
           icon = $i.attr(dataIcon),
        content = $i.attr(dataContent) || $i.html(),
         timeId = Math.random(),
        GMarker, GMarkerOption;

        if ( lat === '' || lng === '' || typeof lat === 'undefined' || typeof lng === 'undefined' ) { return true; }

        GMarkerOption = $.extend({
            'position'  : new google.maps.LatLng(lat, lng),
            'map'       : GMap
        }, settings.option_marker);

        if ( icon !== '' ) { GMarkerOption.icon = icon; }

        GMarker = new google.maps.Marker(GMarkerOption);

        google.maps.event.addListener(GMarker, 'click', function() {
            settings.markerClickZoom && self.markerZoom(GMarker, settings.zoomToLevel);
            settings.markerClickInfo && self.infoWindow(GMarker, content);
        });

        self.markers[timeId] = { Obj: GMarker, content: content };

        if ( minLat > lat || minLat == null ) { minLat = lat; }
        if ( minLng > lng || minLng == null ) { minLng = lng; }
        if ( maxLat < lat || maxLat == null ) { maxLat = lat; }
        if ( maxLng < lng || minLng == null ) { maxLng = lng; }

        $i.attr(dataMark, timeId);
    });

    $items.parent().delegate($items.get(0).tagName, 'click', function() {
        var timeId = $(this).attr(dataMark),
            marker;

        if ( typeof self.markers[timeId] !== 'undefined' ) {
            marker = self.markers[timeId];
            settings.itemClickZoom && self.markerZoom(marker.Obj, settings.zoomToLevel);
            settings.itemClickInfo && self.infoWindow(marker.Obj, marker.content);
        }
    });

    $(settings.mark_reset).bind('click', function() {
        self.fitBounds(minLat, minLng, maxLat, maxLng);
        return false;
    });

    self.fitBounds(minLat, minLng, maxLat, maxLng);

    return this;
};

LatlngMapper.prototype.fitBounds = function(minLat, minLng, maxLat, maxLng) {
    var GMap     = this.GMap,
        GInfoWin = this.GInfoWin;

    GInfoWin.close();
    GMap.fitBounds(
        new google.maps.LatLngBounds(
            new google.maps.LatLng(minLat, minLng),
            new google.maps.LatLng(maxLat, maxLng)
        )
    );
};

LatlngMapper.prototype.infoWindow = function(GMarker, content) {
    var GMap     = this.GMap,
        GInfoWin = this.GInfoWin;

    GMap.panTo(GMarker.getPosition());
    GInfoWin.setOptions({
        'content'        : content
    });
    GInfoWin.open(GMap, GMarker);
    return false;
};

LatlngMapper.prototype.markerZoom = function(GMarker, zoom) {
    var GMap = this.GMap;

    GMap.panTo(GMarker.getPosition());
    GMap.setZoom(zoom);
    return false;
};

})(jQuery);